/*
 * Copyright (c) 2019, ams AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "sns_tmf8801_sensor.h"

extern tof_cmd_data_t  default_tof_cmd_data;
/**
 * Publish all Sensor attributes
 *
 * @param[i] this    reference to this Sensor
 *
 * @return none
 */
static void
tmf8801_publish_prox_attributes(sns_sensor *const this, sns_std_sensor_stream_type stream_type)
{
  ams_deviceInfo_t info;
  int32_t hwid = this->cb->get_registration_index(this);
  if (stream_type == SNS_STD_SENSOR_STREAM_TYPE_STREAMING)
  {
      AMS_DEV_PRINTF(LOW, this, "AMS: tmf8801_publish_prox_attributes, hwid: %d, streaming", hwid);
  }
  else if (stream_type == SNS_STD_SENSOR_STREAM_TYPE_ON_CHANGE)
  {
      AMS_DEV_PRINTF(LOW, this, "AMS: tmf8801_publish_prox_attributes, hwid: %d, on-change", hwid);
  }

  tmf8801_getDeviceInfo(&info, hwid);

  ams_state *state = (ams_state*)this->state->state;
  {
    sns_std_attr_value_data values[] = {SNS_ATTR};

    sns_std_attr_value_data range1[] = {SNS_ATTR, SNS_ATTR};
    range1[0].has_flt = true;
    range1[0].flt = info.tofSensor.rangeMin;
    range1[1].has_flt = true;
    range1[1].flt = info.tofSensor.rangeMax;
    values[0].has_subtype = true;
    values[0].subtype.values.funcs.encode = sns_pb_encode_attr_cb;
    values[0].subtype.values.arg =
      &((pb_buffer_arg){ .buf = range1, .buf_len = ARR_SIZE(range1) });
    sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_RANGES,
        values, ARR_SIZE(values), false);
  }
  {
    sns_std_attr_value_data values[] = {SNS_ATTR};

    sns_std_attr_value_data range1[] = {SNS_ATTR, SNS_ATTR};
    range1[0].has_flt = true;
    range1[0].flt = info.tofSensor.rangeMin;
    range1[1].has_flt = true;
    range1[1].flt = info.tofSensor.rangeMax;
    values[0].has_subtype = true;
    values[0].subtype.values.funcs.encode = sns_pb_encode_attr_cb;
    values[0].subtype.values.arg =
      &((pb_buffer_arg){ .buf = range1, .buf_len = ARR_SIZE(range1) });
    sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_SELECTED_RANGE,
        values, ARR_SIZE(values), false);
  }
  {
    sns_std_attr_value_data values[] = {SNS_ATTR};
    values[0].has_sint = true;
    values[0].sint = stream_type;
    sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_STREAM_TYPE,
        values, ARR_SIZE(values), false);
  }
  {
   sns_std_attr_value_data values[] = {SNS_ATTR};
   values[0].has_sint = true;
   values[0].sint = info.tofSensor.activeCurrent_uA;
   sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_ACTIVE_CURRENT,
       values, ARR_SIZE(values), false);
  }
  {
   sns_std_attr_value_data values[] = {SNS_ATTR};
   values[0].has_sint = true;
   values[0].sint = info.tofSensor.standbyCurrent_uA;
   sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_SLEEP_CURRENT,
       values, ARR_SIZE(values), false);
  }
  if (stream_type == SNS_STD_SENSOR_STREAM_TYPE_STREAMING)
  {
    sns_std_attr_value_data values[] = {SNS_ATTR, SNS_ATTR, SNS_ATTR, SNS_ATTR, SNS_ATTR};

    values[0].has_flt = true;
    values[0].flt = AMS_DEVICE_ODR_1;
    values[1].has_flt = true;
    values[1].flt = AMS_DEVICE_ODR_2;
    values[2].has_flt = true;
    values[2].flt = AMS_DEVICE_ODR_5;
    values[3].has_flt = true;
    values[3].flt = AMS_DEVICE_ODR_10;
    values[4].has_flt = true;
    values[4].flt = AMS_DEVICE_ODR_20;

    state->odr_for_selftest = AMS_DEVICE_ODR_20;

    sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_RATES,
        values, ARR_SIZE(values), false);
  } else {
    /* _ATTRID_RATES not supported; this is an on-change sensor */
    state->odr_for_selftest = 0.0;
  }

  {
    sns_std_attr_value_data values[] = {SNS_ATTR};
    char const op_mode1[] = AMS_OPMODE_NORMAL;

    values[0].str.funcs.encode = pb_encode_string_cb;
    values[0].str.arg = &((pb_buffer_arg)
        { .buf = op_mode1, .buf_len = sizeof(op_mode1) });

    sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_OP_MODES,
        values, ARR_SIZE(values), false);
  }
  {
    sns_std_attr_value_data values[] = {SNS_ATTR};
    char const proto1[] = "sns_proximity.proto";

    values[0].str.funcs.encode = pb_encode_string_cb;
    values[0].str.arg = &((pb_buffer_arg)
        { .buf = proto1, .buf_len = sizeof(proto1) });
    sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_API,
        values, ARR_SIZE(values), false);
  }
  {
    char const name[] = STRINGIFY(CONFIG_AMS_PART_NAME);
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.str.funcs.encode = pb_encode_string_cb;
    value.str.arg = &((pb_buffer_arg)
        { .buf = name, .buf_len = sizeof(name) });
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_NAME, &value, 1, false);
  }
  {
      /*
       * NOTE: Any attributes that are strings must not be null terminated
       * because the at some point in the SEE framework this data is
       * converted to JSON and the JSON parser will throw an error if
       * it encounters NULL characters in a string.
       */

    char const type[] = "proximity";
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.str.funcs.encode = pb_encode_string_cb;
    value.str.arg = &((pb_buffer_arg)
        { .buf = type, .buf_len = sizeof(type) });
    sns_publish_attribute(this, SNS_STD_SENSOR_ATTRID_TYPE, &value, 1, false);
  }
  {
    char const vendor[] = "amsAG";
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.str.funcs.encode = pb_encode_string_cb;
    value.str.arg = &((pb_buffer_arg)
        { .buf = vendor, .buf_len = sizeof(vendor) });
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_VENDOR, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.has_boolean = true;
    value.boolean = false;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_AVAILABLE, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.has_boolean = true;
    value.boolean = false;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_DYNAMIC, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.has_boolean = true;
    value.boolean = (stream_type == SNS_STD_SENSOR_STREAM_TYPE_ON_CHANGE) ? true : false;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_DRI, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.has_boolean = true;
    value.boolean = false;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_STREAM_SYNC, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.has_sint = true;
    value.sint = AMS_DRIVER_VERSION;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_VERSION, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.has_sint = true;
    value.sint = SNS_STD_SENSOR_RIGID_BODY_TYPE_DISPLAY;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_RIGID_BODY, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.has_sint = true;
    value.sint = hwid;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_HW_ID, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    sns_proximity_event stream_event = sns_proximity_event_init_default;
    pb_get_encoded_size(&state->encoded_event_len,
                        sns_proximity_event_fields,
                        &stream_event);
    value.has_sint = true;
    value.sint = state->encoded_event_len;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_EVENT_SIZE, &value, 1, false);
  }
  {
    sns_std_attr_value_data value = sns_std_attr_value_data_init_default;
    value.has_boolean = true;
    value.boolean = true;
    sns_publish_attribute(
        this, SNS_STD_SENSOR_ATTRID_PHYSICAL_SENSOR, &value, 1, true);
  }
}

/* See sns_sensor::init */
static sns_rc _common_tmf8801_prox_init(sns_sensor *const this, sns_std_sensor_stream_type stream_type)
{
  ams_state *state = (ams_state*)this->state->state;
  struct sns_service_manager *smgr= this->cb->get_service_manager(this);
  state->diag_service = (sns_diag_service *)smgr->get_service(smgr, SNS_DIAG_SERVICE);
  state->scp_service =
     (sns_sync_com_port_service *)smgr->get_service(smgr, SNS_SYNC_COM_PORT_SERVICE);
  sns_diag_service* diag = state->diag_service;
  sns_sensor_uid * p_suid;
  int32_t hwid = this->cb->get_registration_index(this);

  state->sensorCmdData = default_tof_cmd_data;
  /* systemData has to be obtained through registry */
  state->sensorSystemData.calReceived = false;
  state->sensorSystemData.stateReceived = false;
  /* our private wrapped-up copies of the com port's handles etc. */
  state->port_handle.scp_service = state->scp_service;
  /* state->port_handle.scp_handle not available until we call
  scp_service->api->sns_scp_register_com_port() */
  state->port_handle_ptr = &state->port_handle;
  // for debug use - allows called fn to call sensor_printf
  state->port_handle.diag = diag;

  //calculate time related parameters
  sysTimeRes = sns_get_time_tick_resolution();
  usec_100_in_ticks = sns_convert_ns_to_ticks(100000);
  usec_10000_in_ticks = usec_100_in_ticks*100;
  usec_5000_in_ticks = usec_100_in_ticks*50;
  usec_1000_in_ticks = usec_100_in_ticks*10;

  if (stream_type == SNS_STD_SENSOR_STREAM_TYPE_ON_CHANGE)
  {
    p_suid = (sns_sensor_uid *)tmf8801_get_suid_with_sensortype_and_hwid(AMS_PROX_OC, hwid);
    state->sensor_type = AMS_PROX_OC;
    sns_memscpy(&state->my_suid,
                sizeof(state->my_suid),
                p_suid,
                sizeof(sns_sensor_uid));
  }
  else
  {
    p_suid = (sns_sensor_uid *)tmf8801_get_suid_with_sensortype_and_hwid(AMS_PROX_STRM, hwid);
    state->sensor_type = AMS_PROX_STRM;
    sns_memscpy(&state->my_suid,
                sizeof(state->my_suid),
                p_suid,
                sizeof(sns_sensor_uid));
  }

  tmf8801_publish_prox_attributes(this, stream_type);

  AMS_DEV_PRINTF(LOW, this
             , "AMS: _common_tmf8801_prox_init: this %p, stream_type %d: state %p, scp_service %p"
             , this
             , stream_type
             , state
             , state->scp_service
             );

  tmf8801_send_suid_req(this, "interrupt", sizeof("interrupt"));
  tmf8801_send_suid_req(this, "timer", sizeof("timer"));
  tmf8801_send_suid_req(this, "registry", sizeof("registry"));

  return SNS_RC_SUCCESS;
}
sns_rc tmf8801_prox_init(sns_sensor *const this)
{
#ifdef ENABLE_DEBUG
  int32_t hwid = this->cb->get_registration_index(this);
  AMS_PRINTF(HIGH, this, "AMS: tmf8801_prox_init:  hwid: %d", hwid);
#endif
  return _common_tmf8801_prox_init(this, SNS_STD_SENSOR_STREAM_TYPE_ON_CHANGE);
}
sns_rc tmf8801_prox_streaming_init(sns_sensor *const this)
{
#ifdef ENABLE_DEBUG
  int32_t hwid = this->cb->get_registration_index(this);
  AMS_PRINTF(HIGH, this, "AMS: tmf8801_prox_streaming_init: hwid: %d", hwid);
#endif
  return _common_tmf8801_prox_init(this, SNS_STD_SENSOR_STREAM_TYPE_STREAMING);
}

sns_rc tmf8801_prox_deinit(sns_sensor *const this)
{
  UNUSED_VAR(this);
  return SNS_RC_SUCCESS;
}

