#pragma once
/*
 * Copyright (c) 2019, ams AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __SNS_tmf8801_SENSOR___H__
#define __SNS_tmf8801_SENSOR___H__

#include "string.h"
#include <limits.h>
#include "msg.h"
#include "pb_decode.h"
#include "pb_encode.h"
#include "sns_async_com_port.pb.h"
#include "sns_async_com_port_pb_utils.h"
#include "sns_attribute_util.h"
#include "sns_com_port_types.h"
#include "sns_data_stream.h"
#include "sns_diag.pb.h"
#include "sns_diag_service.h"
#include "sns_event_service.h"
#include "sns_interrupt.pb.h"
#include "sns_math_util.h"
#include "sns_mem_util.h"
#include "sns_proximity.pb.h"
#include "sns_pb_util.h"
#include "sns_physical_sensor_test.pb.h"
#include "sns_pwr_rail_service.h"
#include "sns_rc.h"
#include "sns_register.h"
#include "sns_request.h"
#include "sns_sensor.h"
#include "sns_sensor_event.h"
#include "sns_sensor_instance.h"
#include "sns_sensor_uid.h"
#include "sns_sensor_util.h"
#include "sns_service.h"
#include "sns_service_manager.h"
#include "sns_std.pb.h"
#include "sns_std_event_gated_sensor.pb.h"
#include "sns_std_sensor.pb.h"
#include "sns_stream_service.h"
#include "sns_suid.pb.h"
#include "sns_sync_com_port_service.h"
#include "sns_time.h"
#include "sns_timer.pb.h"
#include "sns_cal.pb.h"
#include "sns_types.h"
#include "sns_registry_util.h"
#include "sns_island_service.h"
#include "sns_tmf8801_version.h"
#include "sns_tmf8801_log.h"


#ifdef ENDIAN_LITTLE
#define AMS_ENDIAN_1    0
#define AMS_ENDIAN_2    8
#else
#define AMS_ENDIAN_2    0
#define AMS_ENDIAN_1    8
#endif
#ifndef ULLONG_MAX
#define ULLONG_MAX        ((uint64_t)-1)
#endif

/**
 * For turning a compiler option value into a string
 */
#define STRINGIFY(s) STRINGIFY2(s)
#define STRINGIFY2(s) #s
#define AMS_REGISTRY_ENTRY_NAME(s) STRINGIFY(CONFIG_AMS_PART_NAME)s

#define tmf8801_NUM_DEVICES  1

/**
 * define TMF8801 registers
 */
enum tof8801_regs {
  TOF8801_REG_APP_ID            = 0x00,
  TOF8801_REG_MAJ_VER           = 0x01,
  TOF8801_REG_REQ_APP_ID        = 0x02,
  TOF8801_REG_CMD_DAT9          = 0x06,
  TOF8801_REG_CMD_DAT8          = 0x07,
  TOF8801_REG_CMD_DAT7          = 0x08,
  TOF8801_REG_CMD_DAT6          = 0x09,
  TOF8801_REG_CMD_DAT5          = 0x0A,
  TOF8801_REG_CMD_DAT4          = 0x0B,
  TOF8801_REG_CMD_DAT3          = 0x0C,
  TOF8801_REG_CMD_DAT2          = 0x0D,
  TOF8801_REG_CMD_DAT1          = 0x0E,
  TOF8801_REG_CMD_DAT0          = 0x0F,
  TOF8801_REG_CMD               = 0x10,
  TOF8801_REG_MIN_VER           = 0x12,
  TOF8801_REG_STATE             = 0x1C,
  TOF8801_REG_STATUS            = 0x1D,
  TOF8801_REG_DATA_SEC_START    = 0x20,
  TOF8801_REG_ENABLE            = 0xE0,
  TOF8801_REG_INT_STAT          = 0xE1,
  TOF8801_REG_INT_EN            = 0xE2,
  TOF8801_REG_DVICE_ID          = 0xE3,
  TOF8801_REG_REV_ID            = 0xE4
};

enum tof8801_cmd {
  TOF8801_CMD_START_CAP_WITH_DOWNLAOD            = 0x02,
  TOF8801_CMD_START_CAP_NO_DOWNLAOD              = 0x03,
  TOF8801_CMD_START_CALIBRATE                    = 0x0A,
  TOF8801_CMD_DOWNLAOD_NO_CAPTURE                = 0x0B,
  TOF8801_CMD_OSC_TRIM                           = 0x29, // switch to clock trim mode
  TOF8801_CMD_STOP_CAPTURE                       = 0xFF
};

enum tof8801_cmd_data_length {
  TOF8801_CMD_MEASURE_WITH_DOWNLAOD_PARAMS_SIZE         = 0x08,
  TOF8801_CMD_MEASURE_NO_DOWNLAOD_PARAMS_SIZE           = 0x07,
  TOF8801_CMD_DOWNLAOD_NO_MEASURE_PARAMS_SIZE           = 0x01,
};

enum tof8801_interrupt_en {
  TOF8801_HISTOGRAM_INT_EN         = 0x02,
  TOF8801_MEASUREMENT_INT_EN       = 0x01,
};

enum tof8801_standby_ops {
  CPU_STANDBY = 0x00,
  CPU_WAKEUP  = 0x01,
};

enum tof8801_app0_state {
    APP0_IDLE_STATE = 0x01,
};

enum tof8801_app_id {
  TOF8801_APP_ID_BOOTLOADER = 0x80,
  TOF8801_APP_ID_APP1       = 0xC1,
  TOF8801_APP_ID_APP0       = 0xC0,
};

enum tof8801_BL_regs {
  TOF8801_BL_CMD_STAT      = 0x08,
  TOF8801_DATA_SIZE     = 0x09,
  TOF8801_DATA_0        = 0x0A,
  TOF8801_DATA_127      = 0x89,
  TOF8801_CHKSUM        = 0x8A,
};

enum tof8801_bootloader_cmd {
  BL_RESET          = 0x10,
  BL_RAMREMAP_RESET = 0x11,
  BL_UPLOAD_INIT    = 0x14,
  BL_R_RAM          = 0x40,
  BL_W_RAM          = 0x41,
  BL_ADDR_RAM       = 0x43,
};

enum tof8801_bootloader_cmd_stat {
  READY          = 0x0,
  ERR_SIZE       = 0x1,
  ERR_CSUM       = 0x2,
  ERR_RES        = 0x3,
  ERR_APP        = 0x4,
  ERR_TIMEOUT    = 0x5,
  ERR_LOCK       = 0x6,
  ERR_RANGE      = 0x7,
  ERR_MORE       = 0x8,
  ERROR1         = 0x9,
  ERROR2         = 0xA,
  ERROR3         = 0xB,
  ERROR4         = 0xC,
  ERROR5         = 0xD,
  ERROR6         = 0xE,
  ERROR7         = 0xF,
  CMD_BUSY       = 0x10,
  MAX_BL_CMD_STAT,
};

enum tof8801_default_cmdData {
    TMF8801_CMD_ITERATION_DEFAULT  = 0xFF,
    TMF8801_CMD_PERIOD_DEFAULT     = 0x1,
    TMF8801_CMD_THRESHOLD_DEFAULT  = 0,
    TMF8801_CMD_DELAY_DEFAULT      = 0,
    TMF8801_CMD_GPIO_DEFAULT       = 0,
    TMF8801_CMD_ALG_DEFAULT        = 0,
    TMF8801_CMD_DATA_REGISTRY_NUM  = 0x6
};

#define TOF8801_STAT_CPU_STANDBY(x)         ((x) == 0)
#define TOF8801_STAT_CPU_SLEEP(x)           (!((x) & 0x1))
#define TOF8801_STAT_CPU_BUSY(x)            ((x) == 0x1)
#define TOF8801_STAT_CPU_READY(x)           ((x) == 0x41)
#define TOF8801_DEVICE_IS_VALID(x)          ((x & 0x3f) == AMS_DEVICE_ID)
#define CPU_PON0_MASK                      (0x01)
#define RELIABILITY_MASK                    0x3F
#define SYSDATA_DOWNLOAD_CALIB_ONLY         0x1
#define SYSDATA_DOWNLOAD_CALIB_STATE        0x3
#define AMS_DEVICE_ID       0x07
#define AMS_DEVICE_ID_MASK  0x3F
#define AMS_REV_ID          0x00
#define AMS_REV_ID_MASK     0x07

#define I2C_READ  0
#define I2C_WRITE 1
#define AMS_OSC_TRIM_DOWN   (-1)
#define AMS_OSC_TRIM_UP     (1)
#define TMF8801_CALIBRATION_DATA_SIZE       14
#define TMF8801_ALG_STATE_DATA_SIZE         11
#define TMF8801_MEASUREMENT_DATA_SIZE       8
#define TMF8801_ALG_STATE_STRING_SIZE       ((TMF8801_ALG_STATE_DATA_SIZE<<1) + 1)
#define TMF8801_CALIBRATION_STRING_SIZE     ((TMF8801_CALIBRATION_DATA_SIZE<<1) + 1)
#define TMF8801_CAPTURE_BLOCK_SIZE          (TMF8801_MEASUREMENT_DATA_SIZE + TMF8801_ALG_STATE_DATA_SIZE)
//need to start reading from 0x1D, to get timestamp updated and read correctly. read extra 3 bytes 0x1D, 0x1E, 0x1F
#define TMF8801_CAPTURE_TIMESTAMP_BLOCK_SIZE  (TMF8801_MEASUREMENT_DATA_SIZE + TMF8801_ALG_STATE_DATA_SIZE + 3)
#ifdef AMS_TOF_INCLUDE_CONFIG_DATA
#define TMF8801_CONFIG_DATA_SIZE            120
#endif
#define TMF8801_CMD_PARAMS_SIZE             10
#define TMF8801_CMD_TOTAL_SIZE              (TMF8801_CMD_PARAMS_SIZE + 1)
#define TMF8801_CMD_IDX                     (TMF8801_CMD_TOTAL_SIZE - 1)

/**
 * bootloader macro
 */
#define BL_DEFAULT_SALT                    (0x29)
#define BL_IS_CMD_BUSY(x)                  ((x) > 0xF)
#define BL_CMD_WAIT_MSEC                   (1)
#define BL_VALID_CHKSUM                    (0xFF)
#define BL_I2C_CMD_SIZE                    (1)
#define BL_I2C_DATA_LEN_SIZE               (1)
#define BL_I2C_CHKSUM_SIZE                 (1)
#define BL_I2C_NUM_DATA                    (128)
#define BL_I2C_MAX_DATA_SIZE               (BL_I2C_NUM_DATA*sizeof(uint8_t))
#define BL_I2C_HEADER_SIZE                 (BL_I2C_CMD_SIZE + \
                                            BL_I2C_DATA_LEN_SIZE)
#define BL_I2C_FOOTER_SIZE                 (BL_I2C_CHKSUM_SIZE)
#define BL_I2C_CMD_MAX_SIZE                (BL_I2C_HEADER_SIZE   + \
                                            BL_I2C_MAX_DATA_SIZE     + \
                                            BL_I2C_FOOTER_SIZE)
#define BL_CALC_CHKSUM_SIZE(datasize)      ((datasize) + \
                                            BL_I2C_HEADER_SIZE)
#define BL_CALC_BL_CMD_SIZE(datasize)      (BL_CALC_CHKSUM_SIZE(datasize) + \
                                            BL_I2C_FOOTER_SIZE)
#define BL_CALC_BL_RSP_SIZE(datasize)      ((datasize) + \
                                            BL_I2C_HEADER_SIZE + \
                                            BL_I2C_FOOTER_SIZE)

/**
 * supported intel hex record types
 */
#define INTEL_HEX_TYPE_DATA                     0
#define INTEL_HEX_TYPE_EOF                      1
#define INTEL_HEX_TYPE_EXT_LIN_ADDR             4
#define INTEL_HEX_TYPE_START_LIN_ADDR           5
#define INTEL_HEX_BYTE_COUNT_INDEX  0
#define INTEL_HEX_ADDR_HIGH_INDEX   1
#define INTEL_HEX_ADDR_LOW_INDEX    2
#define INTEL_HEX_TYPE_INDEX        3
#define INTEL_HEX_DATA_0_INDEX      4
#define INTEL_HEX_DATA_1_INDEX      5
#define INTEL_HEX_MAX_BYTES_PER_LINE 21

/**
 * clock correction and osc trim constants
 */
#define APP0_MAX_DIST_MM                   (2625) // max dist is 2.5m + 5% error
#define APP0_EXPECTEDRATIO                 (5)
#define APP0_EXP_FREQ_RATIO_Q15            (6972)
#define APP0_MAX_FREQ_RATIO_Q15            (7212) // lowest Frequency
#define APP0_MIN_FREQ_RATIO_Q15            (6732) // highest Frequency
#define APP0_FREQ_RATIO_HITH_Q15           (6991) //12.5 KHz band tolerance
#define APP0_FREQ_RATIO_LOTH_Q15           (6953)
#define APP0_DEFAULT_CLK_ITERATIONS        (15)
#define APP0_CR_WRAPAROUND                 (0x100000000L)      // 64 bits
#define APP0_CR_WRAPAROUND_DIV2            (0x80000000L )      // 32 bits
#define APP0_RATIO_MINCOUNT                (3)

/**
 * AMS customary device ODR (Hz) definitions
 * Note 1 Hz entry is polled mode only - many devices can't interrupt that slow
 * #if ! CONFIG_AMS_USE_DRI - the test sensor is hardcoded to request 1 Hz so
 *                            let's support it even in DRI mode
 */
#define AMS_DEVICE_ODR_1                 1.0
#define AMS_DEVICE_ODR_2                 2.0
#define AMS_DEVICE_ODR_5                 5.0
#define AMS_DEVICE_ODR_10                10.0
#define AMS_DEVICE_ODR_15                15.0
#define AMS_DEVICE_ODR_20                20.0

#define AMS_OPMODE_NORMAL    "NORMAL"

#define AMS_POWER_RAIL_OFF_TIMEOUT_NS   1000000000ULL /* 1 second */
#define AMS_DEVICE_OFF_TO_IDLE_MS       10
#define TOF8801_MAX_STARTUP_RETRY       5
#define AMS_CALIBRATION_DONE           (-1)


typedef enum ams_power_rail_pending_state
{
  AMS_POWER_RAIL_PENDING_NONE,
  AMS_POWER_RAIL_PENDING_INIT,
  AMS_POWER_RAIL_PENDING_SET_CLIENT_REQ,
  AMS_POWER_RAIL_WAITING_RAM_PATCH,
  AMS_POWER_RAIL_PENDING_OFF
} ams_power_rail_pending_state;

typedef enum tof_system_data_version{
    TOF_FAC_CAL_VERSION,
    TOF_STATE_DATA_VERSION,
    CALIBRATION_DATA_INDEX_MAX
}tof_system_data_version;

typedef enum _deviceIdentifier_e {
    AMS_UNKNOWN_DEVICE,
    AMS_TMF8801,
    AMS_LAST_DEVICE
} tof_deviceIdentifier_e;

typedef enum {
    AMS_CONFIG_PROX,
    AMS_CONFIG_GLOBAL,
    AMS_CONFIG_FEATURE_LAST
}ams_configureFeature_t;

typedef enum {
    MODE_OFF            = (0),
    MODE_PROX           = (1),
    MODE_UNKNOWN    /* must be in last position */
} ams_mode_t;

typedef enum ams_tof_dyn_state_t {
    PRX_STATE_UNKNOWN,
    PRX_STATE_IDLE,
    PRX_STATE_CAPTURE,
    PRX_STATE_FACTORY_CALIBRATION,
    PRX_STATE_LAST
}ams_tof_dyn_state_t;

typedef enum _config_options {
    AMS_CONFIG_ENABLE,
    AMS_CONFIG_ALWAYS_READ,
    AMS_CONFIG_ALLOW_INTS,      /* tell dev whether OS has connected the IRQ line */
    AMS_CONFIG_OPTION_LAST
}deviceConfigOptions_t;

typedef enum _report_type {
    AMS_REPORT_NONE        = 0x00,
    AMS_REPORT_ON_CHANGE   = 0x01,
    AMS_REPORT_STREAMING   = 0x02,
    AMS_REPORT_OC_AND_STRM = 0x03,
    AMS_REPORT_TYPE_LAST
}ams_reportType_t;

typedef enum _sensorType {
    AMS_NO_SENSOR_AVAILABLE,
    AMS_AMBIENT_SENSOR,
    AMS_PROXIMITY_SENSOR,
    AMS_ALS_RGB_GAIN_CHANGED,
    AMS_TOF_SENSOR,
    AMS_LAST_SENSOR
}ams_sensorType_t;

typedef struct{
    uint8_t                 deviceId;
    uint8_t                 deviceIdMask;
    uint8_t                 deviceRef;
    uint8_t                 deviceRefMask;
    tof_deviceIdentifier_e  device;
}tof_deviceIdentifier_t;

typedef struct AMS_PORT_portHndl_s {
    sns_sync_com_port_service   *scp_service;
    sns_sync_com_port_handle    scp_handle;
    // for debug use:
    sns_diag_service            *diag;
} AMS_PORT_portHndl;

typedef struct{
    uint8_t alg;
    uint8_t gpio;
    uint8_t delay;
    uint8_t threshold;
    uint8_t period;
    uint8_t iteration[2];
}tof_cmd_data_t;

typedef struct{
    uint8_t calData[TMF8801_CALIBRATION_DATA_SIZE];
}tof_calibration_data_t;

typedef struct{
    uint8_t algStateData[TMF8801_ALG_STATE_DATA_SIZE];
}tof_alg_state_data_t;

#ifdef AMS_TOF_INCLUDE_CONFIG_DATA
typedef struct{
    uint8_t alg_state_data[TMF8801_CONFIG_DATA_SIZE];
}tof_config_data_t;
#endif

typedef struct{
    tof_calibration_data_t  calBlock;
    tof_alg_state_data_t    stateBlock;
#ifdef AMS_TOF_INCLUDE_CONFIG_DATA
    tof_config_data_t       configBlock;
#endif
    uint8_t                 calReceived;
    uint8_t                 stateReceived;
    uint8_t                 ramRegReceived;
}tof_system_data_t;

typedef struct _sensorInfo{
    uint32_t    standbyCurrent_uA;
    uint32_t    activeCurrent_uA;
    uint32_t    rangeMin;
    uint32_t    rangeMax;
    uint16_t    maxPolRate;
    uint8_t     adcBits;
    float_t     resolution;
    char *      driverName;
} ams_sensorInfo_t;

typedef struct _deviceInfo {
    ams_sensorInfo_t tofSensor;
    ams_sensorType_t tableSubSensors[10];
    uint8_t          numberOfSubSensors;
    char             deviceModel[8];
    char *           deviceName;
}ams_deviceInfo_t;

typedef struct {
    uint8_t     capIndex;
    uint8_t     reliability;
    uint16_t    distance;
    uint32_t    timeStamp;
    uint8_t     capStateData[TMF8801_ALG_STATE_DATA_SIZE];
}ams_tof_capture_t;

typedef struct {
    bool        nearBy;
    uint16_t    distance;
    uint8_t     reliability;
    uint32_t    timeStamp;
    uint8_t     accuracy;
} ams_apiPrx_t;

typedef struct
{
    uint32_t first_host;
    uint32_t last_host;
    uint32_t first_i2c;
    uint32_t last_i2c;
    uint32_t ratioQ15;
    uint32_t count;
    uint32_t trim_count;
} tof_clk_cr_t;

typedef struct {
    tof_deviceIdentifier_e  deviceId;
    uint8_t                 tof_fwVersion[5];
    AMS_PORT_portHndl *     portHndl;
    ams_tof_dyn_state_t     tofState;
    ams_tof_capture_t       tofCapData;
    tof_system_data_t *     ctxSystemData;
    tof_cmd_data_t *        ctxCmdData;
    tof_clk_cr_t            ctxClkCr;
    bool                    alwaysReadProx;
    uint32_t                updateAvailable;
    bool                    allowIens;
    bool                    enableCapture;
    bool                    enableCal;
    int32_t                 hwid;
    sns_time                sns_usec_stamp;
    bool                    clk_trim_enable;
}ams_deviceCtx_t;

typedef struct {
    uint8_t command;
    uint8_t size;
    uint8_t seed;
    uint8_t chksum;
}tof8801_BL_upload_init_cmd_t;

typedef struct {
    uint8_t command;
    uint8_t size;
    uint8_t addr_lsb;
    uint8_t addr_msb;
    uint8_t chksum;
}tof8801_BL_addr_ram_cmd_t;

typedef struct {
    uint8_t command;
    uint8_t size;
    uint8_t data[BL_I2C_MAX_DATA_SIZE + 1]; /*chksum in flexible position */
}tof8801_BL_write_ram_cmd_t;

typedef struct tof8801_BL_short_cmd {
    uint8_t command;
    uint8_t size;
    uint8_t chksum;
}tof8801_BL_short_cmd_t;

typedef struct
{
    uint32_t ulba; /* ULBA = upper linear base address = higher 16 bits of the 32-bit address */
    uint32_t addr; /* lower 16-bit of the 32-bit address - old intel formats used just 16-bit addresses */
} intelHexAddr_t;

#include "sns_tmf8801_sensor_instance.h"
/** Interrupt Sensor State. */
typedef struct ams_state
{
    void                    *priv;
    sns_data_stream         *reg_data_stream;
    sns_data_stream         *fw_stream;
    sns_data_stream         *timer_stream;
    sns_sensor_uid          reg_suid;
    sns_sensor_uid          irq_suid;
    sns_sensor_uid          timer_suid;
    ams_sensor_type         sensor_type;
    sns_sensor_uid          my_suid;
    ams_com_port_info       com_port_info;
    sns_interrupt_req       irq_config;

    sns_pwr_rail_service    *pwr_rail_service;
    sns_rail_config         rail_config;
    sns_power_rail_state    registry_rail_on_state;

    bool                            tmf8801_hw_is_present;
    ams_power_rail_pending_state    power_rail_pend_state;

    // debug
    sns_diag_service                *diag_service;
    sns_sync_com_port_service       *scp_service;
    size_t                          encoded_event_len;

    AMS_PORT_portHndl               port_handle;
    AMS_PORT_portHndl               *port_handle_ptr;

    // sensor configuration
    bool                            is_dri;
    int64_t                         hardware_id;
    bool                            supports_sync_stream;
    uint8_t                         resolution_idx;
    float                           odr_for_selftest;

    // registry sensor config
    bool                            registry_cfg_received;
    sns_registry_phy_sensor_cfg     registry_cfg;

    // registry sensor platform config
    bool                            registry_pf_cfg_received;
    sns_registry_phy_sensor_pf_cfg  registry_pf_cfg;

    // placement
    bool                            registry_placement_received;
    float                           placement[12];

    // registry system data and command data
    tof_system_data_t               sensorSystemData;
    tof_cmd_data_t                  sensorCmdData;
    uint32_t                        cal_version[CALIBRATION_DATA_INDEX_MAX];
} ams_state;

typedef struct pb_custom_float_parse_arg {
  float     *data_array;
  uint32_t  *version_array;
  uint8_t   *index;
  uint8_t   size;
} pb_custom_float_parse_arg;

typedef struct pb_custom_string_parse_arg {
  uint8_t   *version_array;
  uint32_t  *index;
  uint32_t  size;
} pb_custom_string_parse_arg;

typedef struct pb_custom_ram_fw_parse_arg {
  AMS_PORT_portHndl         *port_handle_ptr;
  uint32_t                  *index;
} pb_custom_ram_fw_parse_arg;

//TOF default config values
typedef struct pb_tof_default_cmd_data_values{
  float_t   default_config_iteration;
  float_t   default_config_period;
  float_t   default_config_threshold;
  float_t   default_config_delay;
  float_t   default_config_gpio;
  float_t   default_config_alg;
  float_t   *data_array;
  uint32_t  *version_array;
  uint8_t   *index;
  uint8_t   size;
  bool      hasIteration;
  bool      hasPeriod;
  bool      hasThreshold;
  bool      hasDelay;
  bool      hasGpio;
  bool      hasAlg;
} pb_tof_default_cmd_data_values;

sns_time sysTimeRes;
sns_time usec_100_in_ticks;
sns_time usec_10000_in_ticks;
sns_time usec_5000_in_ticks;
sns_time usec_1000_in_ticks;

sns_sensor_api tmf8801_prox_sensor_api;
sns_sensor_api tmf8801_prox_streaming_sensor_api;
tof_deviceIdentifier_e tmf8801_validateDevice(AMS_PORT_portHndl * portHndl);
bool tmf8801_getDeviceInfo(ams_deviceInfo_t * info, int32_t hwid);
uint8_t tmf8801_deviceInit(ams_deviceCtx_t * ctx, AMS_PORT_portHndl * portHndl, tof_system_data_t * sysDat, \
                         tof_cmd_data_t * cmdDat, tof_deviceIdentifier_e chipId, int32_t hwid, uint8_t* fw_ver);
bool tmf8801_deviceEventHandler(sns_sensor_instance *const this, ams_deviceCtx_t * ctx);
uint32_t tmf8801_getUpdateResult(ams_deviceCtx_t * ctx);
bool tmf8801_deviceSetConfig(sns_sensor_instance *const this, ams_deviceCtx_t * ctx, ams_configureFeature_t feature, deviceConfigOptions_t option, uint32_t data);
bool ams_deviceGetConfig(ams_deviceCtx_t * ctx);
uint8_t tmf8801_deviceCalibrate(sns_sensor_instance *instance, ams_deviceCtx_t * ctx);
void tmf8801_reval_instance_config(sns_sensor *this,sns_sensor_instance *instance,ams_sensor_type sensor_type);
void tmf8801_send_suid_req(sns_sensor *this, char *const data_type,uint32_t data_type_len);
void tmf8801_process_suid_events(sns_sensor *const this);
sns_rc tmf8801_sensor_notify_event(sns_sensor *const this);
sns_rc tmf8801_streaming_sensor_notify_event(sns_sensor *const this);
sns_sensor_instance* tmf8801_set_client_request(sns_sensor *const this,
                                           struct sns_request const *exist_request,
                                           struct sns_request const *new_request,
                                           bool remove);
void tmf8801_start_power_rail_timer(sns_sensor *const this,
                                sns_time timeout_ticks,
                                ams_power_rail_pending_state pwr_rail_pend_state);
void tmf8801_set_self_test_inst_config(sns_sensor *this,sns_sensor_instance *instance);
sns_rc tmf8801_prox_init(sns_sensor *const this);
sns_rc tmf8801_prox_streaming_init(sns_sensor *const this);
sns_rc tmf8801_prox_deinit(sns_sensor *const this);
void tmf8801_sensor_send_registry_request(sns_sensor *const this,char *reg_group_name);
void tmf8801_publish_available(sns_sensor *const this);
sns_sensor_uid const* tmf8801_get_prox_oc_sensor_uid(sns_sensor const *const this);
sns_sensor_uid const* tmf8801_get_prox_strm_sensor_uid(sns_sensor const *const this);
sns_sensor_uid const* tmf8801_get_suid_with_sensortype_and_hwid(ams_sensor_type type, int64_t hwid);
uint8_t AMS_tmf8801_rwBlock(AMS_PORT_portHndl * portHndl, uint8_t reg_addr, uint8_t * data, uint8_t length, bool isWrite);
uint8_t AMS_tmf8801_setByte(AMS_PORT_portHndl * portHndl, uint8_t reg_addr, uint8_t data);
void * amsLibrary_open_tmf8801(AMS_PORT_portHndl* port_handle_ptr, tof_system_data_t * sysDat, \
                               tof_cmd_data_t * cmdDat, ams_deviceCtx_t *amsLibrary, int32_t hwid);
uint8_t AMS_string_to_bytes(char* str_in, uint8_t str_length, uint8_t* data_out, uint8_t data_length);
uint8_t AMS_bytes_to_string(uint8_t* data, uint8_t data_length,char* output, uint8_t output_length);
void tmf8801_hexDecode_writeRam(AMS_PORT_portHndl * portHndl, uint8_t* data_in);
uint8_t tmf8801_devicePowerUp(AMS_PORT_portHndl * portHndl, sns_sensor *this);
uint8_t tmf8801_download_systemData(AMS_PORT_portHndl * portHndl, tof_system_data_t * sysDat);
uint8_t tmf8801_cpuGoStandby(AMS_PORT_portHndl * portHndl);
uint8_t tmf8801_wakeFromStandby(AMS_PORT_portHndl * portHndl);
uint8_t tof8801_app0WaitForIdle(AMS_PORT_portHndl * portHndl);
#endif //__SNS_tmf8801_SENSOR___H__
