/*
 * Copyright (c) 2019, ams AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "sns_tmf8801_sensor.h"

static const sns_sensor_uid tmf8801_prox_oc_suids[tmf8801_NUM_DEVICES] = {
    { .sensor_uid = { 0x99, 0x47, 0x8d, 0x81, 0x64, 0x85, 0x4c, 0x56, 0x87, 0xfc, 0x8a, 0xc4, 0xfd, 0xe3, 0xdf, 0x0a }}
};
static const sns_sensor_uid tmf8801_prox_st_suids[tmf8801_NUM_DEVICES] = {
    { .sensor_uid = { 0x09, 0xc1, 0x4a, 0x8a, 0xd8, 0x5b, 0x43, 0x3d, 0xa1, 0x5c, 0x4e, 0xe4, 0x2b, 0xd5, 0x60, 0xb1 }}
};

sns_sensor_uid const* tmf8801_get_prox_oc_sensor_uid(sns_sensor const *const this)
{
  uint32_t hwid = this->cb->get_registration_index(this);
  return tmf8801_get_suid_with_sensortype_and_hwid(AMS_PROX_OC, hwid);

}

sns_sensor_uid const* tmf8801_get_prox_strm_sensor_uid(sns_sensor const *const this)
{
  uint32_t hwid = this->cb->get_registration_index(this);
  return tmf8801_get_suid_with_sensortype_and_hwid(AMS_PROX_STRM, hwid);

}

sns_sensor_uid const* tmf8801_get_suid_with_sensortype_and_hwid(ams_sensor_type type, int64_t hwid)
{
    if (type == AMS_PROX_OC)
    {
        return &tmf8801_prox_oc_suids[hwid];
    }
    else if (type == AMS_PROX_STRM)
    {
        return &tmf8801_prox_st_suids[hwid];
    }
    else
    {
        /* Should not happen, but we need to return something non-null */
        return &tmf8801_prox_oc_suids[hwid];
    }
}

sns_sensor_api tmf8801_prox_sensor_api =
{
  .struct_len         = sizeof(sns_sensor_api),
  .init               = &tmf8801_prox_init,
  .deinit             = &tmf8801_prox_deinit,
  .get_sensor_uid     = &tmf8801_get_prox_oc_sensor_uid,
  .set_client_request = &tmf8801_set_client_request,
  .notify_event       = &tmf8801_sensor_notify_event,
};
sns_sensor_api tmf8801_prox_streaming_sensor_api =
{
  .struct_len         = sizeof(sns_sensor_api),
  .init               = &tmf8801_prox_streaming_init,
  .deinit             = &tmf8801_prox_deinit,
  .get_sensor_uid     = &tmf8801_get_prox_strm_sensor_uid,
  .set_client_request = &tmf8801_set_client_request,
  .notify_event       = &tmf8801_streaming_sensor_notify_event,
};